from typing import List, Optional
from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.crud import todo as todo_crud
from app.db.session import get_db
from app.schemas.todo import Todo, TodoCreate, TodoUpdate

router = APIRouter()


@router.get("/", response_model=List[Todo])
def read_todos(
    skip: int = 0,
    limit: int = 100,
    completed: Optional[bool] = None,
    db: Session = Depends(get_db),
):
    """
    Retrieve todos with optional filtering.
    
    - **skip**: Number of todos to skip (pagination)
    - **limit**: Maximum number of todos to return (pagination)
    - **completed**: Filter by completion status (None for all todos)
    """
    todos = todo_crud.get_todos(db, skip=skip, limit=limit, completed=completed)
    return todos


@router.post("/", response_model=Todo, status_code=status.HTTP_201_CREATED)
def create_todo(todo: TodoCreate, db: Session = Depends(get_db)):
    """
    Create a new todo item.
    
    - **todo**: Todo item to create
    """
    return todo_crud.create_todo(db=db, todo=todo)


@router.get("/{todo_id}", response_model=Todo)
def read_todo(todo_id: int, db: Session = Depends(get_db)):
    """
    Get a specific todo by ID.
    
    - **todo_id**: ID of the todo to retrieve
    """
    db_todo = todo_crud.get_todo(db, todo_id=todo_id)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.put("/{todo_id}", response_model=Todo)
def update_todo(todo_id: int, todo: TodoUpdate, db: Session = Depends(get_db)):
    """
    Update a todo item.
    
    - **todo_id**: ID of the todo to update
    - **todo**: Updated todo data
    """
    db_todo = todo_crud.update_todo(db, todo_id=todo_id, todo=todo)
    if db_todo is None:
        raise HTTPException(status_code=404, detail="Todo not found")
    return db_todo


@router.delete("/{todo_id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
def delete_todo(todo_id: int, db: Session = Depends(get_db)):
    """
    Delete a todo item.
    
    - **todo_id**: ID of the todo to delete
    """
    success = todo_crud.delete_todo(db, todo_id=todo_id)
    if not success:
        raise HTTPException(status_code=404, detail="Todo not found")
    return None